import time
import logging

try:
    from decorator import decorator
except ImportError:
    from .compat import decorator

logging_logger = logging.getLogger(__name__)


def retry(exceptions=Exception, tries=-1, delay=0, backoff=1, logger=logging_logger):
    """Return a retry decorator.

    :param exceptions: an exception or a tuple of exceptions to catch. default: Exception.
    :param tries: the maximum number of attempts. default: -1 (infinite).
    :param delay: initial delay between attempts. default: 0.
    :param backoff: multiplier applied to delay between attempts. default: 1 (no backoff).
    :param logger: logger.warning(fmt, error, delay) will be called on failed attempts.
                   default: retry.logging_logger. if None, logging is disabled.
    """

    @decorator
    def retry_decorator(f, *args, **kwargs):
        _tries, _delay = tries, delay
        while _tries:
            _tries -= 1
            try:
                return f(*args, **kwargs)
            except exceptions as e:
                if not _tries:
                    raise
                if logger is not None:
                    logger.warning('%s, retrying in %s seconds...', e, _delay)
            time.sleep(_delay)
            _delay *= backoff

    return retry_decorator



import functools


def decorator(caller):
    """ Turns caller into a decorator.
    Unlike decorator module, function signature is not preserved.

    :param caller: caller(f, *args, **kwargs)
    """
    def decor(f):
        @functools.wraps(f)
        def wrapper(*args, **kwargs):
            return caller(f, *args, **kwargs)
        return wrapper
    return decor



__all__ = ['retry']

import logging

from .api import retry


# Set default logging handler to avoid "No handler found" warnings.
try:  # Python 2.7+
    from logging import NullHandler
except ImportError:
    class NullHandler(logging.Handler):

        def emit(self, record):
            pass

log = logging.getLogger(__name__)
log.addHandler(NullHandler())



from setuptools import setup


setup(
    name='retry',
    version='0.7.0',
    description='retry decorator',
    long_description=open('README.rst').read(),
    url='https://github.com/invl/retry',
    packages=['retry'],
    license='Apache License 2.0',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        "Programming Language :: Python :: Implementation :: PyPy",
    ]
)
